"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MemodMessage = exports.Message = void 0;
const react_1 = require("@chakra-ui/react");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const chat_1 = require("@strata-foundation/chat");
const react_2 = require("@strata-foundation/react");
const spl_utils_1 = require("@strata-foundation/spl-utils");
const moment_1 = __importDefault(require("moment"));
const react_3 = __importStar(require("react"));
const react_async_hook_1 = require("react-async-hook");
const bs_1 = require("react-icons/bs");
const _1 = require(".");
const __1 = require("..");
const contexts_1 = require("../../contexts");
const hooks_1 = require("../../hooks");
const MessageBody_1 = require("./MessageBody");
const MessageToolbar_1 = require("./MessageToolbar");
const defaultOptions = {
    allowedTags: ["b", "i", "em", "strong", "a", "code", "ul", "li", "p"],
    allowedAttributes: {
        a: ["href", "target"],
    },
};
function Message(props) {
    const { id: messageId, getDecodedMessage, sender, readPermissionAmount, chatKey, txids, startBlockTime, htmlAllowlist = defaultOptions, reacts, type: messageType, showUser = true, pending = false, reply, scrollToMessage, } = props;
    const { publicKey } = (0, wallet_adapter_react_1.useWallet)();
    const { referenceMessageId: emojiReferenceMessageId, showPicker } = (0, contexts_1.useEmojis)();
    const { info: profile } = (0, hooks_1.useWalletProfile)(sender);
    const { info: chatPermissions } = (0, hooks_1.useChatPermissionsFromChat)(chatKey);
    const time = (0, react_3.useMemo)(() => {
        if (startBlockTime) {
            const t = new Date(0);
            t.setUTCSeconds(startBlockTime);
            return t;
        }
    }, [startBlockTime]);
    const readMint = chatPermissions === null || chatPermissions === void 0 ? void 0 : chatPermissions.readPermissionKey;
    const mintAcc = (0, react_2.useMint)(readMint);
    const { metadata } = (0, react_2.useTokenMetadata)(readMint);
    const tokenAmount = mintAcc &&
        readPermissionAmount &&
        (0, spl_utils_1.humanReadable)(readPermissionAmount, mintAcc);
    const { amount: ownedAmount } = (0, hooks_1.useChatOwnedAmount)(publicKey || undefined, chatKey);
    const notEnoughTokens = (0, react_3.useMemo)(() => {
        return (readPermissionAmount &&
            mintAcc &&
            (ownedAmount || 0) < (0, spl_utils_1.toNumber)(readPermissionAmount, mintAcc));
    }, [readPermissionAmount, mintAcc, ownedAmount]);
    // Re decode if not enough tokens changes
    const getDecodedMessageOrIdentity = (_) => getDecodedMessage ? getDecodedMessage() : Promise.resolve(undefined);
    const { result: message, loading: decoding, error: decodeError, } = (0, react_async_hook_1.useAsync)(getDecodedMessageOrIdentity, [notEnoughTokens]);
    const lockedColor = (0, react_1.useColorModeValue)("gray.400", "gray.600");
    const highlightedBg = (0, react_1.useColorModeValue)("gray.200", "gray.800");
    const { handleErrors } = (0, react_2.useErrorHandler)();
    const { sendMessage, error } = (0, contexts_1.useSendMessage)();
    handleErrors(error, decodeError);
    const handleOnReaction = (0, react_3.useCallback)(() => {
        showPicker(messageId);
    }, [showPicker, messageId]);
    const { replyMessage } = (0, contexts_1.useReply)();
    const bg = (0, react_3.useMemo)(() => messageId === emojiReferenceMessageId || messageId === (replyMessage === null || replyMessage === void 0 ? void 0 : replyMessage.id)
        ? highlightedBg
        : "initial", [highlightedBg, emojiReferenceMessageId, messageId, replyMessage === null || replyMessage === void 0 ? void 0 : replyMessage.id]);
    const hover = (0, react_3.useMemo)(() => ({ bg: highlightedBg }), [highlightedBg]);
    const textColor = (0, react_1.useColorModeValue)("black", "white");
    const loadingSkeleton = (0, react_3.useMemo)(() => {
        return (react_3.default.createElement(react_1.Skeleton, { startColor: lockedColor, height: "20px" }, Array.from({ length: genLength(messageId || "") }, () => ".").join()));
    }, [messageId, lockedColor]);
    const buyMoreTrigger = (0, react_3.useCallback)((props) => {
        return (react_3.default.createElement(react_1.Tooltip, { label: `You need ${tokenAmount} ${metadata === null || metadata === void 0 ? void 0 : metadata.data.symbol} to read this message` },
            react_3.default.createElement(react_1.HStack, { onClick: props.onClick, spacing: 2, _hover: { cursor: "pointer" } },
                react_3.default.createElement(react_1.Skeleton, { startColor: lockedColor, height: "20px", speed: 100000 }, Array.from({ length: genLength(messageId || "") }, () => ".").join()),
                react_3.default.createElement(react_1.Icon, { color: lockedColor, as: bs_1.BsLockFill }))));
    }, [tokenAmount, metadata, lockedColor, messageId]);
    // LEGACY: If this is a reaction before message types were stored on the top level instead of json
    if ((message === null || message === void 0 ? void 0 : message.type) === chat_1.MessageType.React) {
        return null;
    }
    return (react_3.default.createElement(react_1.Box, { position: "relative", _hover: hover, bg: bg },
        react_3.default.createElement(react_1.Popover, { matchWidth: true, trigger: "hover", placement: "top-end", isLazy: true },
            react_3.default.createElement(react_1.PopoverContent, { w: "full", bg: "transparent", border: "none" },
                react_3.default.createElement(react_1.PopoverBody, null,
                    react_3.default.createElement(MessageToolbar_1.MessageToolbar, Object.assign({}, props)))),
            react_3.default.createElement(react_1.PopoverTrigger, null,
                react_3.default.createElement(react_1.VStack, { spacing: 0, gap: 0, w: "full" },
                    reply && (react_3.default.createElement(_1.DisplayReply, { reply: reply, scrollToMessage: scrollToMessage })),
                    react_3.default.createElement(react_1.HStack, { pl: 2, pr: 2, pb: 1, pt: reply ? 0 : 1, w: "full", align: "start", spacing: 2, className: "strata-message" },
                        showUser ? (react_3.default.createElement(react_1.Avatar, { mt: "6px", size: "sm", src: profile === null || profile === void 0 ? void 0 : profile.imageUrl })) : (react_3.default.createElement(react_1.Box, { w: "34px" })),
                        react_3.default.createElement(react_1.VStack, { w: "full", align: "start", spacing: 0 },
                            showUser && (react_3.default.createElement(_1.MessageHeader, { chatKey: chatKey, sender: sender, startBlockTime: startBlockTime })),
                            react_3.default.createElement(react_1.Box, { w: "fit-content", position: "relative", textAlign: "left", wordBreak: "break-word", color: textColor, id: messageId }, message && messageType ? (react_3.default.createElement(MessageBody_1.MessageBody, { htmlAllowlist: htmlAllowlist, message: message, messageType: messageType })) : decoding ? (loadingSkeleton) : notEnoughTokens ? (react_3.default.createElement(__1.BuyMoreButton, { mint: readMint, trigger: buyMoreTrigger })) : (react_3.default.createElement(react_1.Tooltip, { label: `Failed to decode message` },
                                react_3.default.createElement(react_1.Skeleton, { startColor: lockedColor, height: "20px", speed: 100000 }, Array.from({ length: genLength(messageId || "") }, () => ".").join())))),
                            reacts && reacts.length > 0 && (react_3.default.createElement(_1.Reacts, { onAddReaction: handleOnReaction, reacts: reacts, onReact: (emoji, mine) => {
                                    if (!mine)
                                        sendMessage({
                                            message: {
                                                type: chat_1.MessageType.React,
                                                emoji: emoji,
                                                referenceMessageId: messageId,
                                            },
                                        });
                                } }))),
                        react_3.default.createElement(react_1.HStack, { alignItems: "center", flexShrink: 0 },
                            showUser && (react_3.default.createElement(react_1.Hide, { below: "md" },
                                react_3.default.createElement(react_1.Text, { fontSize: "xs", color: "gray.500", _dark: { color: "gray.400" } }, (0, moment_1.default)(time).format("LT")))),
                            react_3.default.createElement(_1.MessageStatus, { txids: txids, pending: pending }))))))));
}
exports.Message = Message;
const lengths = {};
function genLength(id) {
    if (!lengths[id]) {
        lengths[id] = 10 + Math.random() * 100;
    }
    return lengths[id];
}
exports.MemodMessage = react_3.default.memo(Message);
//# sourceMappingURL=Message.js.map